function [T,data] = AgentModelerV2(aNo,aFds,tFinal,NetFunc,InterFunc,varargin)
% AgentModelerV2 runs an agent-based simulation with a definied network
% structure and interaction rules.
%       
%   AgentModelerV2 creates the intial agent states and, if wanted, creates
%   the timesteps which will be recorded in the simulation. With this
%   information the function feed it into a definied network function as
%   well as a definied interaction function. The function will then return
%   the results of the network function.
%
%   [T,DATA] = AgentModelerV2(N,PROPS,T,NETWORK,INTERACT,NAME1,VALUE1, ...)
%   first creates N total agents with properties defined in PROPS. PROPS is
%   defined as so:
%
%       PROPS = { propName1 propType1 values1 weights1;
%                 propName2 propType2 values2 weights2;
%                 ...                                 }
%
%       propName - Name of the property as a string.
%
%       propType - What type of property it is, being either:
%
%           single_value - a single floating point numbers
%
%           vector_value - a vector of floating point numbers on every
%           other agent.
%
%       values - a cell or single value containing the initial value for
%       the prop. E.g. { a b c } will create agents with three different
%       values a, b and c. Values can be:
%
%           a - which will make agents have the same initial value a.
%
%           [a b] - which generate a random value for each agent on a
%           uniform distribution on the interval [a b].
%
%       weights - a vector of values defining how many agents will have a  
%       particular initialvalue. E.g. if values is {a b c} and weights is
%       [2 3 1] then 2/6 agents will initialise, with a 3/6 with initialise
%       with b, and 1/6 will initialise with c.
%
%   T can be a single integer or a vector. If T is a single integer then
%   the function will create a vector of time value to record agent states
%   at. If it is a vector then T agent states will be recorded at each
%   state of T. NETWORK is the function that drives the network structure
%   of the sim and is expected to be: 
%
%       DATA = NETWORK(AGENTS,T,INTERACT) where AGENTS are the initial 
%       state of the agetns, T is the vector of recording timesteps, 
%       INTERACT is an interaction function, and DATA is a cell containing
%       the state of each agent at each recording timestep.
%
%   INTERACT is a function that controls individual agent interaction and
%   is expected to be:
%
%       [AGENTI,AGENTJ] = INTERACT(AGENTI,AGENTJ) where AGENTI is the the
%       first agent in the interaction and AGENTJ is the second agent.
%
%   NAME and VALUE corespond to teriary values in for AgentModelerV2. These
%   include:
%
%       'identify' which is a boolean that determines if agents will be
%       given an id number.
%
%       't_spacing' which is a positive integer greater than 1 which is how
%       many interaction happen before the agent states are recorded.

    [idfy, T] = InitialiseVar(varargin, tFinal);%Define extra variables
    agents = InitialiseAgents(aNo, aFds, idfy);%Generate initial agent states
    
    data = NetFunc(agents,T,InterFunc);%Run the model
    
end


function [idfy, T] = InitialiseVar(set, tFinal)
    
    idfy   = true;%Default idfy
    if length(tFinal) == 1 %if t records isn't defined
        tSpace = max([round(tFinal*10^-3),1]);%picking a suitable t spacing
    end
    
    for i = 1:2:length(set) %parsing user input
        switch set{i}
            case 'identify'
                idfy   = set{i+1};
            case 't_spacing'
                if set{i+1} < 1
                    error('t Spacing is less than 1')
                end
                if length(tFinal) < 1
                    warning('t Space is implicit with tFinal being a vector');
                end
                tSpace = set{i+1};
            otherwise
                warning(['Unreconised setting "' set{i} '"'])
        end
    end
    
    if length(tFinal) == 1 %If t records isn't defined
        T = 0:tSpace:tFinal; %Generate t recorded
        if T(length(T)) ~= tFinal
            T = [T tFinal]; %Make sure the ending time is what the user spesified
        end
    else
        T = tFinal; %otherwise set t recorded as user's
    end
end

function agents = InitialiseAgents(aNo, aFds, idfy)
    if size(aFds,2) < 3 %Check if information about agents are missing
        error('information about agents are missing')
    end    
    spesifiedWeights = size(aFds,2) >= 4; %are the weights spesified

    if idfy %generate id if wanted
        agents(aNo,1).id = aNo;
        for aIdx = 1:(aNo-1)
            agents(aIdx).id = aIdx;
        end
    else
        agents(aNo,1).(aFds{1}) = []; %otherwise just predefine struc
    end
      
    for fdsIdx = 1:size(aFds,1) %defining initial states
        %% How many different types of agents
        if ~iscell(aFds{fdsIdx,3})
            propGroups = 1;
        else
            propGroups = length(aFds{fdsIdx,3}); 
        end
        %% defining the weights of those groups
        if propGroups > 1            
            if ~spesifiedWeights || isempty(aFds{fdsIdx,4})                
                weights = ones(propGroups); %Undefinined weight means equal weighting
            else
                if length(aFds{fdsIdx,4}) ~= propGroups
                    error('Weights are given to undefined property groups')
                end
                weights = aFds{fdsIdx,4};
            end
            totalWeight = sum(weights); %Need the total weight of the weights
        end
        
        %% Determine the nature of the agent state
        switch aFds{fdsIdx,2}
            case 'single_value'
                if propGroups == 1 %All agents are from the same group so define them the same 
                    for agentIdx = 1:aNo
                        agents(agentIdx).(aFds{fdsIdx,1}) = DefineProp(aFds{fdsIdx,3});
                    end
                else
                    aDefined = 0;% keep track of the already defined agent
                    for propIdx = 1:(propGroups-1) % over all of the defined groups but last
                        
                        % find when to terminate
                        aPortion = round(aNo*weights(propIdx)/totalWeight); 
                        
                        %Define the agents al the way up to that point
                        for agentIdx = (1:aPortion)+aDefined 
                            agents(agentIdx).(aFds{fdsIdx,1}) = DefineProp(aFds{fdsIdx,3}{propIdx});
                        end
                        aDefined = agentIdx;
                    end
                    
                    %Define the last set of agents
                    for agentIdx = (aDefined+1):aNo
                        agents(agentIdx).(aFds{fdsIdx,1}) = DefineProp(aFds{fdsIdx,3}{propGroups});
                    end
                end
                
                
%             case 'single_value_on_interveal'    
%                 for agentIdx = 1:aNo
%                     agents(agentIdx).(aFds{fdsIdx,1}) = (rand*diff(aFds{fdsIdx,3}) + aFds{fdsIdx,3}(1));
%                 end
                
            case 'vector_value'
                if ~idfy
                    error('You cannot have vector values for agent properties while they have no id')
                end
                
                for agentIdx = 1:aNo
                    agents(agentIdx).(aFds{fdsIdx,1}) = DefineProp(aFds{fdsIdx,3})* ones(aNo,1);
                end
            otherwise
                error(['Unreconised initial value type "' aFds{fdsIdx,2} '"'])
                
        end       
    end
end

function prop = DefineProp(setting)
    if length(setting) == 1
        prop = setting;
    else
        prop = rand*diff(setting) + setting(1);
    end
end

