function [agenti,agentj] = BayesInterFixedP(agenti,agentj,p)
% BayesInteractionFixedP governs the interaction of two agents according to
% an extend model of (Martins 2009).
%
% [agenti,agentj] = BayesInteractionFixedP(agenti,agentj,p)
%
% Inputs:   agenti, a structure representing the initial state of the first
%           agent.
%               agenti.x is the opinion of the agent.
%               agenti.sig is the uncertainty of the agent.
%
%           agentj, a structure representing the initial state of the 
%           second agent.
%               agentj.x is the opinion of the agent.
%               agentj.sig is the uncertainty of the agent.
%
%           p, the global misinformation rate between 0 and 1.
%
% Outputs:  agenti, a structure representing the final state of the first
%           agent.
%               agenti.x is the opinion of the agent.
%               agenti.sig is the uncertainty of the agent.
%
%           agentj, a structure representing the final state of the second 
%           agent.
%               agentj.x is the opinion of the agent.
%               agentj.sig is the uncertainty of the agent.
%
% NOTE: To use BayesInteractionFixedP in AgentModeler, 
% BayesInteractionFixedP needs to put through an anonymous function such
% that,
% AnonFunc =  @(agenti,agentj) BayesInteractionFixedP(agenti,agentj,p);
% where p is a predefined variable. AnonFunc can then be used as input into
% AgentModeler
    
%% Quick evaulation
    if (agenti.sig == 0) && (agentj.sig == 0)
        return %If both uncertainties are 0 no change        
    end

%% Initialising
    %Defining old opinions and uncertainties
    xi = agenti.x;     xj = agentj.x;
    sigi = agenti.sig; sigj = agentj.sig;
    
%% Convergence tests for values
    % p*
    pPhi = p*exp(-(xi-xj)^2/2/(sigi^2+sigj^2))/sqrt(2*pi*(sigi^2+sigj^2));
    pStar = pPhi/(pPhi+1-p);

    % ratio of agents' uncertainty 
    rSig = sigj/sigi;

    % normalised uncertainty of i
    rSigP1divde1 = sigi^2/(sigi^2+sigj^2);

%% Agent Opinion Updating    
    %Agent i's weighted opinion
    Wi = xi/(1+(1/rSig)^2);
    
    %Agent j's weighted opinion
    Wj = xj/(1+(rSig)^2);
    
    %The agent compromise
    Comp = pStar * (Wi+Wj);

    %Agents updated opinions
    agenti.x = Comp + (1-pStar)*xi;
    agentj.x = Comp + (1-pStar)*xj;

%% Agent Uncertianty Updating
   
    agenti.sig = sqrt( sigi^2*(1 - pStar*rSigP1divde1) +  pStar*(1-pStar)*((xi-xj)/(1+(rSig)^2))^2 );
    agentj.sig = sqrt( sigj^2*(1 - pStar*(1-rSigP1divde1)) +  pStar*(1-pStar)*((xi-xj)/(1+(1/rSig)^2))^2 );
    
    