function [agenti,agentj] = BayesInterMartinsFixedP(agenti,agentj,p)
% BayesInteractionFixedP governs the interaction of two agents according to
% an extend model of (Martins 2009).
%
% [agenti,agentj] = BayesInteractionFixedP(agenti,agentj,p)
%
% Inputs:   agenti, a structure representing the initial state of the first
%           agent.
%               agenti.x is the opinion of the agent.
%               agenti.sig is the uncertainty of the agent.
%
%           agentj, a structure representing the initial state of the 
%           second agent.
%               agentj.x is the opinion of the agent.
%               agentj.sig is the uncertainty of the agent.
%
%           p, the global misinformation rate between 0 and 1.
%
% Outputs:  agenti, a structure representing the final state of the first
%           agent.
%               agenti.x is the opinion of the agent.
%               agenti.sig is the uncertainty of the agent.
%
%           agentj, a structure representing the final state of the second 
%           agent.
%               agentj.x is the opinion of the agent.
%               agentj.sig is the uncertainty of the agent.
%
% NOTE: To use BayesInteractionFixedP in AgentModeler, 
% BayesInteractionFixedP needs to put through an anonymous function such
% that,
% AnonFunc =  @(agenti,agentj) BayesInteractionFixedP(agenti,agentj,p);
% where p is a predefined variable. AnonFunc can then be used as input into
% AgentModeler

%% Quick evaulations
    if (agenti.sig == 0) && (agentj.sig == 0)
        return %If both uncertainties are 0 no change        
    end

%% Initialising
    %Defining old opinions and uncertainties
    xi = agenti.x;     xj = agentj.x;
    sigi = agenti.sig; sigj = agentj.sig;
    
%% Convergence tests for values
    % p*
    pStarI =  pStarFunc(xi-xj,sigi,p);
    
    pStarJ = pStarFunc(xi-xj,sigj,p);
    
%% Agent Opinion Updating 

    agenti.x = pStarI * (xi+xj)/2 + (1-pStarI)*xi;
    agentj.x = pStarJ * (xi+xj)/2 + (1-pStarJ)*xj;

%% Agent Uncertianty Updating
   
    agenti.sig = sqrt( sigi^2*(1 - pStarI/2) +  pStarI*(1-pStarI)*((xi-xj)/2)^2 );
    agentj.sig = sqrt( sigj^2*(1 - pStarJ/2) +  pStarJ*(1-pStarJ)*((xi-xj)/2)^2 );
    
function pStar = pStarFunc(delta,sig,p)
    pPhi = p*exp(-(delta)^2/4/(sig^2))/2/sig/sqrt(pi);
    pStar = pPhi/(pPhi+1-p);

    