function data = PairOffNet(agents,T,interFunc,varargin)
% PairOffNet runs a social simulation where agents are randomly paired off
% and agents interact with a supplied interation rule.
%
%   DATA = PairOffNet(AGENTS,T,INTERACT) will intislise the simultion with
%   AGENTS, where AGENTS is a structure of the intial state of all agents in the
%   simulation. The function will record into DATA the agent states at each
%   interaction number defined in T. The maximum amount of agents will be
%   paired of each interaction cycle. INTERACT is a function that controls 
%   individual agent interaction and is expected to be:
%
%       [AGENTI,AGENTJ] = INTERACT(AGENTI,AGENTJ) where AGENTI is the the
%       first agent in the interaction and AGENTJ is the second agent.
%
%   DATA = PairOffNet(AGENTS,T,INTERACT,INTERACTIONS) is the same as above
%   but only INTERACTIONS pairs of agents will interact each interaction
%   cycle.
    
%% Define Variables
    n	      = length(agents);
    TLength   = length(T);
    fds	      = fieldnames(agents); fdsNo = length(fds);
    TCounter  = 2; %Counting through the recorded cycles
    [intersIntval,isIntval] = DefineInteractionsIntval(n,varargin);
    
%% Running the model
    %predefine data
    data = PredefineData(agents,n,fds,fdsNo,TLength);
       
    for t = 1:T(TLength)
        %% Agent pairs for this cycle
        if isIntval
            %Interaction nubmer is on interval
            pairs = PairOffAgents(randi( intersIntval ),n);
        else
            %Interaction number is fixed
            pairs = PairOffAgents(intersIntval,n);
        end
        
        %% Make the agent pairs interact
        for pIdx = 1:size(pairs,1)
            [agents(pairs(pIdx,1)),agents(pairs(pIdx,2))] = interFunc(agents(pairs(pIdx,1)),agents(pairs(pIdx,2)));
        end
        
        %% Record agent state
        if t == T(TCounter)
            data = WriteToData(data,agents,fds,fdsNo,TCounter);
            
            TCounter = TCounter +1;
        end
    end
            
    
end

function [interactionsIntval,isIntval] = DefineInteractionsIntval(n,userInterval)
    if isempty(userInterval)
        interactionsIntval = floor(n/2);
        return
    end
    switch length(userInterval{1})
        case 2
            if userInterval{1}(1) < 0 || userInterval{1}(2) < 0
                error('The interacts interval is negative. It must be positive!')
            end
            if userInterval{1}(1) > userInterval{1}(2)
                error('The minimum of the non-interacts interval is larger than the maximum.')            
            end
            interactionsIntval = userInterval{1};
            isIntval = true;
        case 1
            if userInterval{1} < 0
                error('Cannot have negative interactions!')
            end
            interactionsIntval = userInterval{1};
            isIntval = false;
        otherwise
            error('Unexpected interaction interval size input')
    end
end

function data = PredefineData(agents,n,fds,fdsNo,TLength)
    
    data = cell(fdsNo-1,1);
    
    skippedId = false;
    for fdsIdx = 1:fdsNo
        
        if ~strcmp('id',fds{fdsIdx})
            data{fdsIdx-skippedId} = [permute([agents.(fds{fdsIdx})],[3 2 1]); ...
                                         zeros(TLength-1, n ,  size([agents.(fds{fdsIdx})],1))];
        else
            skippedId = true;
        end
    end

end

function pairs = PairOffAgents(inters,n)
            
    agentPerm        = randperm(n,2*inters)';
    pairs = [ agentPerm(1:inters) agentPerm((inters+1):(2*inters)) ];
end

function data = WriteToData(data,agents,fds,fdsNo,T)

    skippedId = false;
    for fdsIdx = 1:fdsNo
        if ~strcmp('id',fds{fdsIdx})
            data{fdsIdx-skippedId}(T,:,:) = permute([agents.(fds{fdsIdx})],[3 2 1]);
        else
            skippedId = true;
        end
    end
end


